//
//  Webhooks.swift
//  Telegrammer
//
//  Created by Givi Pataridze on 28.04.2018.
//

import Foundation
import HeliumLogger
import LoggerAPI
import HTTP
import NIO

class Webhooks: Connection {
    
	public var bot: Bot
	public var dispatcher: Dispatcher
	public var worker: Worker
	public var running: Bool
    
    public var readLatency: TimeAmount = .seconds(2)
    public var clean: Bool = false
    public var maxConnections: Int = 40
	
	private var server: HTTPServer?
	
    public init(bot: Bot, dispatcher: Dispatcher, worker: Worker = MultiThreadedEventLoopGroup(numberOfThreads: 4)) {
		self.bot = bot
		self.dispatcher = dispatcher
		self.worker = worker
		self.running = false
	}
	
    public func start() throws -> Future<Void> {
        guard let ip = bot.settings.webhooksIp,
            let url = bot.settings.webhooksUrl,
            let port = bot.settings.webhooksPort,
            let publicCert = bot.settings.webhooksPublicCert,
            let privateKey = bot.settings.webhooksPrivateKey else {
                throw CoreError(identifier: "Webhooks",
                                reason: "Initialization parameters wasn't found in enviroment variables")
        }
        let tlsConfig = TLSConfiguration.forServer(certificateChain: [.file(publicCert)], privateKey: .file(privateKey))
        
        guard let fileHandle = FileHandle(forReadingAtPath: publicCert) else {
            let errorDescription = "Public key '\(publicCert)' for HTTPS server wasn't found"
            Log.error(errorDescription)
            throw CoreError(identifier: "FileIO", reason: errorDescription)
        }
        
        let cert = InputFile(data: fileHandle.readDataToEndOfFile(), filename: publicCert)
        let params = Bot.SetWebhookParams(url: url, certificate: cert, maxConnections: maxConnections, allowedUpdates: nil)
        return try bot.setWebhook(params: params).flatMap { (success) -> Future<Void> in
            Log.info("setWebhook request result: \(success)")
            return try self.listenWebhooks(on: ip, port: port, tlsConfig: tlsConfig).then { $0.onClose }
        }
    }
	
    private func listenWebhooks(on host: String, port: Int, tlsConfig: TLSConfiguration) throws -> Future<HTTPServer> {
        return HTTPServer.start(hostname: host, port: port, responder: dispatcher, tlsConfig: tlsConfig, on: worker)
            .do { (server) in
                Log.info("HTTPS server started on: \(host):\(port)")
                self.server = server
                self.running = true
            }
    }
	
	public func stop() {
		self.running = false
		_ = self.server?.close()
	}
}
